@@ -42,50 +42,6 @@ def direct_tcpip(client, host, port):
     )
 
 
-def get_gateway(host, port, cache, replace=False):
-    """
-    Create and return a gateway socket, if one is needed.
-
-    This function checks ``env`` for gateway or proxy-command settings and
-    returns the necessary socket-like object for use by a final host
-    connection.
-
-    :param host:
-        Hostname of target server.
-
-    :param port:
-        Port to connect to on target server.
-
-    :param cache:
-        A ``HostConnectionCache`` object, in which gateway ``SSHClient``
-        objects are to be retrieved/cached.
-
-    :param replace:
-        Whether to forcibly replace a cached gateway client object.
-
-    :returns:
-        A ``socket.socket``-like object, or ``None`` if none was created.
-    """
-    from fabric.state import env, output
-    sock = None
-    proxy_command = ssh_config().get('proxycommand', None)
-    if env.gateway:
-        gateway = normalize_to_string(env.gateway)
-        # ensure initial gateway connection
-        if replace or gateway not in cache:
-            if output.debug:
-                print "Creating new gateway connection to %r" % gateway
-            cache[gateway] = connect(*normalize(gateway) + (cache, False))
-        # now we should have an open gw connection and can ask it for a
-        # direct-tcpip channel to the real target. (bypass cache's own
-        # __getitem__ override to avoid hilarity - this is usually called
-        # within that method.)
-        sock = direct_tcpip(dict.__getitem__(cache, gateway), host, port)
-    elif proxy_command:
-        sock = ssh.proxycommand(proxy_command)
-    return sock
-
-
 class HostConnectionCache(dict):
     """
     Dict subclass allowing for caching of host connections/clients.
@@ -124,9 +80,25 @@ class HostConnectionCache(dict):
         """
         Force a new connection to ``key`` host string.
         """
+        from fabric.state import env, output
         user, host, port = normalize(key)
         key = normalize_to_string(key)
-        self[key] = connect(user, host, port, cache=self)
+        sock = None
+        proxy_command = ssh_config().get('proxycommand', None)
+        if env.gateway:
+            gateway = normalize_to_string(env.gateway)
+            # Ensure initial gateway connection
+            if gateway not in self:
+                if output.debug:
+                    print "Creating new gateway connection to %r" % gateway
+                self[gateway] = connect(*normalize(gateway))
+            # Now we should have an open gw connection and can ask it for a
+            # direct-tcpip channel to the real target. (Bypass our own
+            # __getitem__ override to avoid hilarity.)
+            sock = direct_tcpip(dict.__getitem__(self, gateway), host, port)
+        elif proxy_command:
+            sock = ssh.ProxyCommand(proxy_command)
+        self[key] = connect(user, host, port, sock)
 
     def __getitem__(self, key):
         """
@@ -199,10 +171,8 @@ def key_filenames():
     # Honor SSH config
     conf = ssh_config()
     if 'identityfile' in conf:
-        # Handle Paramiko 1.9 vs 1.10 treatment of this setting.
-        # 1.9 is a string, 1.10 is a list of strings.
-        value = conf['identityfile']
-        keys.extend([value] if isinstance(value, basestring) else value)
+        # Assume a list here as we require Paramiko 1.10+
+        keys.extend(conf['identityfile'])
     return map(os.path.expanduser, keys)
 
 
@@ -330,23 +300,12 @@ def normalize_to_string(host_string):
     return join_host_strings(*normalize(host_string))
 
 
-def connect(user, host, port, cache, seek_gateway=True):
+def connect(user, host, port, sock=None):
     """
     Create and return a new SSHClient instance connected to given host.
 
-    :param user: Username to connect as.
-
-    :param host: Network hostname.
-
-    :param port: SSH daemon port.
-
-    :param cache:
-        A ``HostConnectionCache`` instance used to cache/store gateway hosts
-        when gatewaying is enabled.
-
-    :param seek_gateway:
-        Whether to try setting up a gateway socket for this connection. Used so
-        the actual gateway connection can prevent recursion.
+    If ``sock`` is given, it's passed into ``SSHClient.connect()`` directly.
+    Used for gateway connections by e.g. ``HostConnectionCache``.
     """
     from state import env, output
 
@@ -357,6 +316,11 @@ def connect(user, host, port, cache, seek_gateway=True):
     # Init client
     client = ssh.SSHClient()
 
+    # Load system hosts file (e.g. /etc/ssh/ssh_known_hosts)
+    known_hosts = env.get('system_known_hosts')
+    if known_hosts:
+        client.load_system_host_keys(known_hosts)
+
     # Load known host keys (e.g. ~/.ssh/known_hosts) unless user says not to.
     if not env.disable_known_hosts:
         client.load_system_host_keys()
@@ -370,22 +334,14 @@ def connect(user, host, port, cache, seek_gateway=True):
 
     # Initialize loop variables
     connected = False
-    password = get_password()
+    password = get_password(user, host, port)
     tries = 0
-    sock = None
 
     # Loop until successful connect (keep prompting for new password)
     while not connected:
         # Attempt connection
         try:
             tries += 1
-
-            # (Re)connect gateway socket, if needed.
-            # Nuke cached client object if not on initial try.
-            if seek_gateway:
-                sock = get_gateway(host, port, cache, replace=tries > 0)
-
-            # Ready to connect
             client.connect(
                 hostname=host,
                 port=int(port),
@@ -462,7 +418,7 @@ def connect(user, host, port, cache, seek_gateway=True):
                 text = prompt % env.host_string
             password = prompt_for_password(text)
             # Update env.password, env.passwords if empty
-            set_password(password)
+            set_password(user, host, port, password)
         # Ctrl-D / Ctrl-C for exit
         except (EOFError, TypeError):
             # Print a newline (in case user was sitting at prompt)
