@@ -2,6 +2,7 @@
 Internal subroutines for e.g. aborting execution with an error message,
 or performing indenting on multiline output.
 """
+import os
 import sys
 import textwrap
 from traceback import format_exc
@@ -248,8 +249,7 @@ def _pty_size():
         import termios
         import struct
 
-    default_rows, default_cols = 24, 80
-    rows, cols = default_rows, default_cols
+    rows, cols = 24, 80
     if not win32 and sys.stdout.isatty():
         # We want two short unsigned integers (rows, cols)
         fmt = 'HH'
@@ -262,11 +262,6 @@ def _pty_size():
                 buffer)
             # Unpack buffer back into Python data types
             rows, cols = struct.unpack(fmt, result)
-            # Fall back to defaults if TIOCGWINSZ returns unreasonable values
-            if rows == 0:
-                rows = default_rows
-            if cols == 0:
-                cols = default_cols
         # Deal with e.g. sys.stdout being monkeypatched, such as in testing.
         # Or termios not having a TIOCGWINSZ.
         except AttributeError:
@@ -356,3 +351,10 @@ class RingBuffer(list):
             raise ValueError("Can't set a slice of a ring buffer!")
         else:
             return self._super.__setitem__(key, value)
+
+
+def apply_lcwd(path, env):
+    # Apply CWD if a relative path
+    if not os.path.isabs(path) and env.lcwd:
+        path = os.path.join(env.lcwd, path)
+    return path
